﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;

namespace HeuristicLab.Analysis.AlgorithmBehavior.Analyzers {
  public static class ConvexHullMeasures {
    //TODO: Calculates only the volumne of a d-simplex
    public static double CalculateVolume(List<double[]> hyperHull) {
      double[,] diffs = new double[hyperHull.Count, hyperHull.Count];

      for (int i = 0; i < hyperHull.Count; i++) {
        for (int j = 0; j < hyperHull.Count; j++) {
          diffs[i, j] = CalculateVectorDiff(hyperHull[i], hyperHull[j]);
        }
      }

      double det = alglib.rmatrixdet(diffs);
      double result = det / hyperHull[0].Count().Fact();

      return result;
    }

    private static double CalculateVectorDiff(double[] vector1, double[] vector2) {
      double result = 0.0;

      for (int i = 0; i < vector1.Length; i++) {
        result += vector1[i] - vector2[i];
      }

      return result;
    }
  }
}
